@@ -1,7 +1,6 @@
 //! Declare DataStream, a type that wraps RawCellStream so as to be useful
 //! for byte-oriented communication.
 
-use super::RawCellStream;
 use crate::{Error, Result};
 use tor_cell::relaycell::msg::EndReason;
 
@@ -18,8 +17,9 @@ use tokio_util::compat::{FuturesAsyncReadCompatExt, FuturesAsyncWriteCompatExt};
 
 use std::io::Result as IoResult;
 use std::pin::Pin;
-use std::sync::Arc;
 
+use crate::circuit::StreamTarget;
+use crate::stream::StreamReader;
 use tor_cell::relaycell::msg::{Data, RelayMsg};
 
 /// An anonymized stream over the Tor network.
@@ -111,21 +111,22 @@ pub struct DataReader {
 }
 
 impl DataStream {
-    /// Wrap a RawCellStream as a DataStream.
+    /// Wrap raw stream reader and target parts as a DataStream.
     ///
-    /// Call only after a CONNECTED cell has been received.
-    pub(crate) fn new(s: RawCellStream) -> Self {
-        let s = Arc::new(s);
+    /// For non-optimistic stream, function `wait_for_connection`
+    /// must be called after to make sure CONNECTED is received.
+    pub(crate) fn new(reader: StreamReader, target: StreamTarget) -> Self {
         let r = DataReader {
             state: Some(DataReaderState::Ready(DataReaderImpl {
-                s: Arc::clone(&s),
+                s: reader,
                 pending: Vec::new(),
                 offset: 0,
+                connected: false,
             })),
         };
         let w = DataWriter {
             state: Some(DataWriterState::Ready(DataWriterImpl {
-                s,
+                s: target,
                 buf: Box::new([0; Data::MAXLEN]),
                 n_pending: 0,
             })),
@@ -137,6 +138,30 @@ impl DataStream {
     pub fn split(self) -> (DataReader, DataWriter) {
         (self.r, self.w)
     }
+
+    /// Wait until a CONNECTED cell is received, or some other cell
+    /// is received to indicate an error.
+    ///
+    /// Does nothing if this stream is already connected.
+    pub(crate) async fn wait_for_connection(&mut self) -> Result<()> {
+        // We must put state back before returning
+        let state = self.r.state.take().expect("Missing state in DataReader");
+
+        if let DataReaderState::Ready(imp) = state {
+            let (imp, result) = if imp.connected {
+                (imp, Ok(()))
+            } else {
+                // This succeeds if the cell is CONNECTED, and fails otherwise.
+                imp.read_cell().await
+            };
+            self.r.state = Some(DataReaderState::Ready(imp));
+            result
+        } else {
+            Err(Error::InternalError(
+                "Expected ready state of a new stream.".to_owned(),
+            ))
+        }
+    }
 }
 
 impl AsyncRead for DataStream {
@@ -208,8 +233,8 @@ enum DataWriterState {
 
 /// Internal: the write part of a DataStream
 struct DataWriterImpl {
-    /// The underlying RawCellStream object.
-    s: Arc<RawCellStream>,
+    /// The underlying StreamTarget object.
+    s: StreamTarget,
 
     /// Buffered data to send over the connection.
     // TODO: this buffer is probably smaller than we want, but it's good
@@ -397,8 +422,8 @@ enum DataReaderState {
 
 /// Wrapper for the read part of a DataStream
 struct DataReaderImpl {
-    /// The underlying RawCellStream object.
-    s: Arc<RawCellStream>,
+    /// The underlying StreamReader object.
+    s: StreamReader,
 
     /// If present, data that we received on this stream but have not
     /// been able to send to the caller yet.
@@ -408,6 +433,9 @@ struct DataReaderImpl {
 
     /// Index into pending to show what we've already read.
     offset: usize,
+
+    /// If true, we have received a CONNECTED cell on this stream.
+    connected: bool,
 }
 
 impl AsyncRead for DataReader {
@@ -506,14 +534,18 @@ impl DataReaderImpl {
         let cell = self.s.recv().await;
 
         let result = match cell {
-            Ok(RelayMsg::Data(d)) => {
+            Ok(RelayMsg::Connected(_)) if !self.connected => {
+                self.connected = true;
+                Ok(())
+            }
+            Ok(RelayMsg::Data(d)) if self.connected => {
                 self.add_data(d.into());
                 Ok(())
             }
             Ok(RelayMsg::End(e)) => Err(Error::EndReceived(e.reason())),
             Err(e) => Err(e),
             Ok(m) => {
-                self.s.protocol_error().await;
+                self.s.protocol_error();
                 Err(Error::StreamProto(format!(
                     "Unexpected {} cell on stream",
                     m.cmd()
